import FWCore.ParameterSet.Config as cms

particleFlowBlock = cms.EDProducer("PFBlockProducer",

    # verbosity 
    verbose = cms.untracked.bool(False),
    # Debug flag
    debug = cms.untracked.bool(False),

    # input clusters
    PFClustersECAL = cms.InputTag("particleFlowClusterECAL"),
    PFClustersHCAL = cms.InputTag("particleFlowClusterHCAL"),
    PFClustersHFEM = cms.InputTag("particleFlowClusterHFEM"),
    PFClustersHFHAD = cms.InputTag("particleFlowClusterHFHAD"),
    PFClustersPS = cms.InputTag("particleFlowClusterPS"),
    EGPhotons = cms.InputTag("photons"),                                   

    # input tracks
    GsfRecTracks = cms.InputTag("pfTrackElec"),
    ConvBremGsfRecTracks = cms.InputTag("pfTrackElec","Secondary"),
    useConvBremGsfTracks = cms.bool(False),                                     
    RecTracks = cms.InputTag("pfTrack"),
    useConvBremPFRecTracks = cms.bool(True),

    # input nuclear interactions 
    PFNuclear = cms.InputTag("pfDisplacedTrackerVertex"),
    useNuclear = cms.bool(True),

    # This parameters defines the level of purity of
    # nuclear interactions choosen.
    # Level 1 is only high Purity sample labeled as isNucl
    # Level 2 isNucl + isNucl_Loose (2 secondary tracks vertices)
    # Level 3 isNucl + isNucl_Loose + isNucl_Kink
    #         (low purity sample made of 1 primary and 1 secondary track)
    # By default the level 1 is teh safest one.

    nuclearInteractionsPurity = cms.uint32(1),                          

    # input muons
    RecMuons = cms.InputTag("muons"),

    # input conversions
    PFConversions = cms.InputTag("pfConversions"),
    useConversions = cms.bool(False),

    # input V0
    PFV0 = cms.InputTag("pfV0"),
    useV0 = cms.bool(False),

    # Track Quality Cut: Tracks are kept if DPt/Pt < sigma * Cut
    # and if nHit >= cut
    pf_DPtoverPt_Cut = cms.vdouble(-1.0,-1.0,-1.0,-1.0,-1.0),
    pf_NHit_Cut = cms.vuint32(3,3,3,3,3),
                                   
    # Run particle flow at HLT (hence no RecMuons, no GSF tracks)
    usePFatHLT = cms.bool(False),

    # Turn of track quality cuts that require iterative tracking for heavy-ions
    useIterTracking = cms.bool(True),

    # Photon selection. Et cut; Track iso (cste;slope), Ecal iso (cste, slope), Hcal iso (cste, slope), H/E
    # just put infinite Et cut to disable the photon import
    useEGPhotons = cms.bool(True),                                   
    PhotonSelectionCuts = cms.vdouble(10,2.0, 0.001, 4.2, 0.003, 2.2, 0.001, 0.05)                                   
)


